Rep = 20000;
ns = [30 60 120 500];
level = .05;
coef = 1;
boot = 1000;
rng(5192021);

nmodel = 5;
nn = length(ns);

vce = 1;

sizeT = zeros(nmodel,nn);
sizeBoot = zeros(nmodel,nn);
sizeBM = zeros(nmodel,nn);

for ni = 1:nn
  n = ns(ni);

for modeli = 1:nmodel;

  xi = (1:n)'/(n+1);
  if modeli == 1
      X1 = [2;2;2;ones(n-3,1)];
  elseif modeli == 2
      X1 = 1./sqrt(1-xi);
  elseif modeli == 3
      X1 = gaminv(xi,1/4,1);
  elseif modeli == 4
      X1 = exp(norminv(xi));
  elseif modeli == 5
      X1 = exp(norminv(xi)*2);
  end
  sig = X1 + 1;
  X = [X1,ones(n,1)];
  Q = inv(X'*X);
  XQ = X*Q;
  k = size(X,2);
  h = sum((X.*(X*Q))')';

  pvaluesT = zeros(Rep,1);
  pvaluesBoot = zeros(Rep,1);
  pvaluesBM = zeros(Rep,1);
  for rep = 1:Rep
    Y = randn(n,1).*sig;
    beta = XQ'*Y; 
    e = Y - X*beta;
    
    if vce == 0
	Xe = XQ.*repmat(e,1,k);
	V = Xe'*Xe;
    elseif vce == 1
	Xe = XQ.*repmat(e,1,k);
	V = (Xe'*Xe)*(n/(n-k));
    elseif vce == 2
	if max(h) < 1
  	   Xe = XQ.*repmat(e./sqrt(1-h),1,k);
	   V = Xe'*Xe;
	else
	   Xe = XQ.*repmat(e,1,k);
	   V = (Xe'*Xe)*(n/(n-k));
	end
    elseif vce == 3
	if max(h) < 1
  	   Xe = XQ.*repmat(e./(1-h),1,k);
	   V = Xe'*Xe;
	else
	   Xe = XQ.*repmat(e,1,k);
	   V = (Xe'*Xe)*(n/(n-k));
	end
    end
    tstat = abs(beta(coef))./sqrt(V(coef,coef));
    bootstore = zeros(boot,1);
    for bi = 1:boot;
	ri = randi(2,n,1)*2 - 3;
	Yb = e.*ri;
	betab = XQ'*Yb;
	eb = Yb - X*betab;
	Xeb = XQ.*repmat(eb,1,k);
	Vb = (Xeb'*Xeb)*(n/(n-k));
	bootstore(bi,1) = abs(betab(coef))./sqrt(Vb(coef,coef));
    end
    pboot = mean(bootstore > tstat);
    pvaluesT(rep,1) = 2*(1-tcdf(tstat,n-k));
    pvaluesBoot(rep,1) = pboot;
    pvaluesBM(rep,1) = 2*(1-BM_cdf(tstat,X,vce,coef));
  end
  sizeT(modeli,ni) = mean(pvaluesT < level);
  sizeBoot(modeli,ni) = mean(pvaluesBoot < level);
  sizeBM(modeli,ni) = mean(pvaluesBM < level);

delete Table1Hetero1.txt;
diary Table1Hetero1.txt;
disp('---------------------')
fprintf('Size: Heteroskedastic Case \r\n');
fprintf('HC: method = %d\r\n', vce);
fprintf('Number of Replications: Rep = %d\r\n', Rep);
fprintf('Number of Bootstraps: Boot = %d\r\n', boot);
fprintf('Nominal Level: alpha = %.2f\r\n', level);
disp('---------------------')
fprintf('Model 1 = Dummy\r\n');
fprintf('Model 2 = Pareto(2)\r\n');
fprintf('Model 3 = Gamma(1/4,1)\r\n');
fprintf('Model 4 = exp(N(0,1))\r\n');
fprintf('Model 5 = exp(N(0,4))\r\n');
disp('---------------------')
fprintf('Sample sizes\r\n');
disp(ns);
disp('---------------------')
fprintf('Student t Distribution\r\n');
disp(sizeT);
disp('---------------------')
fprintf('Bootstrap Distribution\r\n');
disp(sizeBoot);
disp('---------------------')
fprintf('BM Distribution\r\n');
disp(sizeBM);
disp('---------------------')
diary off;

end
end
