Rep = 20000;
ns = [30 60 120 250 500];
level = .05;
coef = 1;
rng(5192021);

nmodel = 5;
nn = length(ns);

vce = 1;

sizeT3 = zeros(nmodel,nn);
sizeT4 = zeros(nmodel,nn);

for ni = 1:nn
  n = ns(ni);

for modeli = 1:nmodel;

  xi = (1:n)'/(n+1);
  if modeli == 1
      X1 = [2;2;2;ones(n-3,1)];
  elseif modeli == 2
      X1 = 1./sqrt(1-xi);
  elseif modeli == 3
      X1 = gaminv(xi,1/4,1);
  elseif modeli == 4
      X1 = exp(norminv(xi));
  elseif modeli == 5
      X1 = exp(norminv(xi)*2);
  end
  sig = 1 + X1;
  X = [X1,ones(n,1)];
  Q = inv(X'*X);
  XQ = X*Q;
  k = size(X,2);

  pvaluesT3 = zeros(Rep,1);
  pvaluesT4 = zeros(Rep,1);
  for rep = 1:Rep
    Y = randn(n,1).*sig;
    beta = XQ'*Y; 
    e = Y - X*beta;
    h = sum((X.*(X*Q))')';
    if vce == 0
	Xe = XQ.*repmat(e,1,k);
	V = Xe'*Xe;
    elseif vce == 1
	Xe = XQ.*repmat(e,1,k);
	V = (Xe'*Xe)*(n/(n-k));
    elseif vce == 2
	if max(h) < 1
  	   Xe = XQ.*repmat(e./sqrt(1-h),1,k);
	   V = Xe'*Xe;
	else
	   Xe = XQ.*repmat(e,1,k);
	   V = (Xe'*Xe)*(n/(n-k));
	end
    elseif vce == 3
	if max(h) < 1
  	   Xe = XQ.*repmat(e./(1-h),1,k);
	   V = Xe'*Xe;
	else
	   Xe = XQ.*repmat(e,1,k);
	   V = (Xe'*Xe)*(n/(n-k));
	end
    end
    tstat = abs(beta(coef))./sqrt(V(coef,coef));
    pvaluesT3(rep,1) = 2*(1-T3cdf(tstat,X,sig,vce,coef));
    pvaluesT4(rep,1) = 2*(1-T4cdf(tstat,X,sig,vce,coef));
  end
  sizeT3(modeli,ni) = mean(pvaluesT3 < level);
  sizeT4(modeli,ni) = mean(pvaluesT4 < level);

delete Table4Hetero1.txt;
diary Table4Hetero1.txt;
disp('---------------------')
fprintf('Size: Heteroskedastic Case \r\n');
fprintf('HC: method = %d\r\n', vce);
fprintf('Number of Replications: Rep = %d\r\n', Rep);
fprintf('Nominal Level: alpha = %.2f\r\n', level);
disp('---------------------')
fprintf('Model 1 = Dummy\r\n');
fprintf('Model 2 = Pareto(2)\r\n');
fprintf('Model 3 = Gamma(1/4,1)\r\n');
fprintf('Model 4 = exp(N(0,1))\r\n');
fprintf('Model 5 = exp(N(0,4))\r\n');
disp('---------------------')
fprintf('Sample sizes\r\n');
disp(ns);
disp('---------------------')
fprintf('Three-Parameter Distribution\r\n');
disp(sizeT3);
disp('---------------------')
fprintf('Four-Parameter Distribution\r\n');
disp(sizeT4);
disp('---------------------')
diary off;

end
end