Rep = 20000;
ns = [30 60 120 250 500];
level = .05;
coef = 1;
rng(5192021);

nmodel = 5;
nn = length(ns);

vce = 3;

sizeFT = zeros(nmodel,nn);

for ni = 1:nn
  n = ns(ni);

for modeli = 1:nmodel;

  xi = (1:n)'/(n+1);
  if modeli == 1
      X1 = [2;2;2;ones(n-3,1)];
  elseif modeli == 2
      X1 = 1./sqrt(1-xi);
  elseif modeli == 3
      X1 = gaminv(xi,1/4,1);
  elseif modeli == 4
      X1 = exp(norminv(xi));
  elseif modeli == 5
      X1 = exp(norminv(xi)*2);
  end
  sig = X1 + 1;
  X = [X1,ones(n,1)];
  Q = inv(X'*X);
  XQ = X*Q;
  k = size(X,2);
  h = sum((X.*(X*Q))')';

  pvaluesFT = zeros(Rep,1);
  for rep = 1:Rep
    Y = randn(n,1).*sig;
    beta = XQ'*Y; 
    e = Y - X*beta;
    
    if vce == 0
	Xe = XQ.*repmat(e,1,k);
	V = Xe'*Xe;
    elseif vce == 1
	Xe = XQ.*repmat(e,1,k);
	V = (Xe'*Xe)*(n/(n-k));
    elseif vce == 2
	if max(h) < 1
  	   Xe = XQ.*repmat(e./sqrt(1-h),1,k);
	   V = Xe'*Xe;
	else
	   Xe = XQ.*repmat(e,1,k);
	   V = (Xe'*Xe)*(n/(n-k));
	end
    elseif vce == 3
	if max(h) < 1
  	   Xe = XQ.*repmat(e./(1-h),1,k);
	   V = Xe'*Xe;
	else
	   Xe = XQ.*repmat(e,1,k);
	   V = (Xe'*Xe)*(n/(n-k));
	end
    end
    tstat = abs(beta(coef))./sqrt(V(coef,coef));
    e2 = (e./(1-h)).^2;
    X2 = [ones(n,1),(X1.^2)];
    [etahat,w,sig2hat,ehat,R2,Cn] = gma(e2,X2,1,1);
    sigt = mean(e2)/100;
    sig2hat = sig2hat.*(sig2hat >= sigt) + sigt.*(sig2hat < sigt);
    sighat = sqrt(sig2hat);
    pvaluesFT(rep,1) = 2*(1-Tcdf(tstat,X,sighat,vce,coef));
  end
  sizeFT(modeli,ni) = mean(pvaluesFT < level);

delete Table6Hetero3.txt;
diary Table6Hetero3.txt;
disp('---------------------')
fprintf('Size: Heteroskedastic Case \r\n');
fprintf('HC: method = %d\r\n', vce);
fprintf('Number of Replications: Rep = %d\r\n', Rep);
fprintf('Nominal Level: alpha = %.2f\r\n', level);
disp('---------------------')
fprintf('Model 1 = Dummy\r\n');
fprintf('Model 2 = Pareto(2)\r\n');
fprintf('Model 3 = Gamma(1/4,1)\r\n');
fprintf('Model 4 = exp(N(0,1))\r\n');
fprintf('Model 5 = exp(N(0,4))\r\n');
disp('---------------------')
fprintf('Sample sizes\r\n');
disp(ns);
disp('---------------------')
fprintf('Feasible Approximate Distribution\r\n');
disp(sizeFT);
disp('---------------------')
diary off;

end
end
