function G = Tcdf(x,X,sig,vce,coef)

maxit = 100000;
tolerance = .0001;

n = size(X,1);
k = size(X,2);
nx = length(x);
Q = inv(X'*X);
XQ = X*Q;
Z = XQ(:,coef);
B0 = Z.^2;
if vce == 0
	B = B0;
else
	B = B0*(n/(n-k));
end
if vce > 1
	h = sum((X.*XQ)')';
	if (max(h) < 1)*(vce == 2)
		B = B0./(1-h);
	elseif (max(h) < 1)*(vce == 3)
		B = B0./((1-h).^2);
	end
end
sigXQ = sig.*XQ;
sigXQR = sigXQ(:,coef);
A1 = sigXQR*(sigXQR');
BX = B.*X;
sigBX = sig.*BX;
A3 = sigBX*(sigXQ');
A2 = diag(B.*(sig.^2)) - A3 - A3' + (sigXQ*(X'*BX))*(sigXQ');

G = zeros(nx,1);
for i = 1:nx
   xi = x(i);
   if abs(xi) > eps
	C = A1 - A2*(xi^2);
	[H1,lambda1] = eigs(C,1,'largestreal');
	C1 = H1*(lambda1*H1') - C;
	C2 = C1'*C1;
	mu1 = sum(diag(C1))/lambda1;
	mu2 = sum(diag(C2))/(lambda1^2);
	mu3 = sum(sum(C2.*C1))/(lambda1^3);
	mu4 = sum(sum(C2.*C2))/(lambda1^4);
	tau = (mu1*mu4 - mu2*mu3)/(mu2*mu4-mu3^2);
	dtau = (mu1*mu3 - mu2^2)/(mu2*mu4-mu3^2);
	tau2 = (tau^2 - 4*dtau)*(tau^2 > 4*dtau);
	a2 = 2/(tau - sqrt(tau2));
	a1 = (mu3 - a2*mu2)/(mu2-mu1*a2);
	nu1 = (mu1*a2 - mu2)/(a1*(a2-a1));
	nu2 = (mu1 - nu1*a1)/a2;
	nu = nu1 + nu2;
	delta = a1/a2;
	if (delta > 1e-6)*(delta < 1);
		c = log(1/(1-delta));
 		t = 1 - (((1/delta-1)*c)^(nu2/2))*tolerance/(1-delta);
		M = ceil((chi2inv(t,nu2)-c*nu2)/(2*c));
	else
		M = maxit+1;
	end

	if (M <= maxit)
		m = (0:M);
		Km = nu + m*2;
		b = nbinpdf(m',nu2/2,delta);
		th = .158/tolerance;
		K1 = Km(Km < th);
		K2 = Km(Km >= th);
		F1 = tcdf(sqrt(a1*K1),K1);
		F2 = normcdf(sqrt(a1*K2));
		F = [F1,F2];
		Gi = F*b;
	else
		a = mu1 - (mu2^2)/mu3;
		b = mu3/mu2;
		r = (mu2^3)/(mu3^2);
		Gi = integral(@(x) fun(x,a,b,r),0,Inf,'RelTol',0,'AbsTol',tolerance);
	end
	G(i) = Gi*(xi > 0) + (1 - Gi)*(xi < 0);
   else
 	G(i) = 0.5;
   end

end

function G = fun(x,a,b,r)
	G = normcdf(sqrt(a+x*b)).*chi2pdf(x,r);
end

end